@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,26 +19,31 @@
 
 """QtWebKit specific part of the web element API."""
 
+import typing
+
 from PyQt5.QtCore import QRect
 from PyQt5.QtWebKit import QWebElement, QWebSettings
+from PyQt5.QtWebKitWidgets import QWebFrame
 
 from qutebrowser.config import config
-from qutebrowser.utils import log, utils, javascript
+from qutebrowser.utils import log, utils, javascript, usertypes
 from qutebrowser.browser import webelem
+MYPY = False
+if MYPY:
+    # pylint: disable=unused-import,useless-suppression
+    from qutebrowser.browser.webkit import webkittab
 
 
 class IsNullError(webelem.Error):
 
     """Gets raised by WebKitElement if an element is null."""
 
-    pass
-
 
 class WebKitElement(webelem.AbstractWebElement):
 
     """A wrapper around a QWebElement."""
 
-    def __init__(self, elem, tab):
+    def __init__(self, elem: QWebElement, tab: 'webkittab.WebKitTab') -> None:
         super().__init__(tab)
         if isinstance(elem, self.__class__):
             raise TypeError("Trying to wrap a wrapper!")
@@ -46,90 +51,94 @@ class WebKitElement(webelem.AbstractWebElement):
             raise IsNullError('{} is a null element!'.format(elem))
         self._elem = elem
 
-    def __str__(self):
+    def __str__(self) -> str:
         self._check_vanished()
         return self._elem.toPlainText()
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         if not isinstance(other, WebKitElement):
             return NotImplemented
         return self._elem == other._elem  # pylint: disable=protected-access
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> str:
         self._check_vanished()
         if key not in self:
             raise KeyError(key)
         return self._elem.attribute(key)
 
-    def __setitem__(self, key, val):
+    def __setitem__(self, key: str, val: str) -> None:
         self._check_vanished()
         self._elem.setAttribute(key, val)
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         self._check_vanished()
         if key not in self:
             raise KeyError(key)
         self._elem.removeAttribute(key)
 
-    def __contains__(self, key):
+    def __contains__(self, key: object) -> bool:
+        assert isinstance(key, str)
         self._check_vanished()
         return self._elem.hasAttribute(key)
 
-    def __iter__(self):
+    def __iter__(self) -> typing.Iterator[str]:
         self._check_vanished()
         yield from self._elem.attributeNames()
 
-    def __len__(self):
+    def __len__(self) -> int:
         self._check_vanished()
         return len(self._elem.attributeNames())
 
-    def _check_vanished(self):
+    def _check_vanished(self) -> None:
         """Raise an exception if the element vanished (is null)."""
         if self._elem.isNull():
             raise IsNullError('Element {} vanished!'.format(self._elem))
 
-    def has_frame(self):
+    def has_frame(self) -> bool:
         self._check_vanished()
         return self._elem.webFrame() is not None
 
-    def geometry(self):
+    def geometry(self) -> QRect:
         self._check_vanished()
         return self._elem.geometry()
 
-    def classes(self):
+    def classes(self) -> typing.List[str]:
         self._check_vanished()
         return self._elem.classes()
 
-    def tag_name(self):
+    def tag_name(self) -> str:
         """Get the tag name for the current element."""
         self._check_vanished()
         return self._elem.tagName().lower()
 
-    def outer_xml(self):
+    def outer_xml(self) -> str:
         """Get the full HTML representation of this element."""
         self._check_vanished()
         return self._elem.toOuterXml()
 
-    def value(self):
+    def value(self) -> webelem.JsValueType:
         self._check_vanished()
         val = self._elem.evaluateJavaScript('this.value')
         assert isinstance(val, (int, float, str, type(None))), val
         return val
 
-    def set_value(self, value):
+    def set_value(self, value: webelem.JsValueType) -> None:
         self._check_vanished()
         if self._tab.is_deleted():
             raise webelem.OrphanedError("Tab containing element vanished")
         if self.is_content_editable():
             log.webelem.debug("Filling {!r} via set_text.".format(self))
+            assert isinstance(value, str)
             self._elem.setPlainText(value)
         else:
             log.webelem.debug("Filling {!r} via javascript.".format(self))
             value = javascript.to_js(value)
             self._elem.evaluateJavaScript("this.value={}".format(value))
 
-    def dispatch_event(self, event, bubbles=False,
-                       cancelable=False, composed=False):
+    def dispatch_event(self, event: str,
+                       bubbles: bool = False,
+                       cancelable: bool = False,
+                       composed: bool = False) -> None:
         self._check_vanished()
         log.webelem.debug("Firing event on {!r} via javascript.".format(self))
         self._elem.evaluateJavaScript(
@@ -140,7 +149,7 @@ class WebKitElement(webelem.AbstractWebElement):
                     javascript.to_js(cancelable),
                     javascript.to_js(composed)))
 
-    def caret_position(self):
+    def caret_position(self) -> int:
         """Get the text caret position for the current element."""
         self._check_vanished()
         pos = self._elem.evaluateJavaScript('this.selectionStart')
@@ -148,7 +157,7 @@ class WebKitElement(webelem.AbstractWebElement):
             return 0
         return int(pos)
 
-    def insert_text(self, text):
+    def insert_text(self, text: str) -> None:
         self._check_vanished()
         if not self.is_editable(strict=True):
             raise webelem.Error("Element is not editable!")
@@ -160,7 +169,7 @@ class WebKitElement(webelem.AbstractWebElement):
             this.dispatchEvent(event);
         """.format(javascript.to_js(text)))
 
-    def _parent(self):
+    def _parent(self) -> typing.Optional['WebKitElement']:
         """Get the parent element of this element."""
         self._check_vanished()
         elem = self._elem.parent()
@@ -168,7 +177,7 @@ class WebKitElement(webelem.AbstractWebElement):
             return None
         return WebKitElement(elem, tab=self._tab)
 
-    def _rect_on_view_js(self):
+    def _rect_on_view_js(self) -> typing.Optional[QRect]:
         """Javascript implementation for rect_on_view."""
         # FIXME:qtwebengine maybe we can reuse this?
         rects = self._elem.evaluateJavaScript("this.getClientRects()")
@@ -180,8 +189,8 @@ class WebKitElement(webelem.AbstractWebElement):
             return None
 
         text = utils.compact_text(self._elem.toOuterXml(), 500)
-        log.webelem.vdebug("Client rectangles of element '{}': {}".format(
-            text, rects))
+        log.webelem.vdebug(  # type: ignore
+            "Client rectangles of element '{}': {}".format(text, rects))
 
         for i in range(int(rects.get("length", 0))):
             rect = rects[str(i)]
@@ -206,7 +215,8 @@ class WebKitElement(webelem.AbstractWebElement):
 
         return None
 
-    def _rect_on_view_python(self, elem_geometry):
+    def _rect_on_view_python(self,
+                             elem_geometry: typing.Optional[QRect]) -> QRect:
         """Python implementation for rect_on_view."""
         if elem_geometry is None:
             geometry = self._elem.geometry()
@@ -220,7 +230,8 @@ class WebKitElement(webelem.AbstractWebElement):
             frame = frame.parentFrame()
         return rect
 
-    def rect_on_view(self, *, elem_geometry=None, no_js=False):
+    def rect_on_view(self, *, elem_geometry: QRect = None,
+                     no_js: bool = False) -> QRect:
         """Get the geometry of the element relative to the webview.
 
         Uses the getClientRects() JavaScript method to obtain the collection of
@@ -250,7 +261,7 @@ class WebKitElement(webelem.AbstractWebElement):
         # No suitable rects found via JS, try via the QWebElement API
         return self._rect_on_view_python(elem_geometry)
 
-    def _is_visible(self, mainframe):
+    def _is_visible(self, mainframe: QWebFrame) -> bool:
         """Check if the given element is visible in the given frame.
 
         This is not public API because it can't be implemented easily here with
@@ -302,8 +313,8 @@ class WebKitElement(webelem.AbstractWebElement):
             visible_in_frame = visible_on_screen
         return all([visible_on_screen, visible_in_frame])
 
-    def remove_blank_target(self):
-        elem = self
+    def remove_blank_target(self) -> None:
+        elem = self  # type: typing.Optional[WebKitElement]
         for _ in range(5):
             if elem is None:
                 break
@@ -313,14 +324,14 @@ class WebKitElement(webelem.AbstractWebElement):
                 break
             elem = elem._parent()  # pylint: disable=protected-access
 
-    def _move_text_cursor(self):
+    def _move_text_cursor(self) -> None:
         if self.is_text_input() and self.is_editable():
             self._tab.caret.move_to_end_of_document()
 
-    def _requires_user_interaction(self):
+    def _requires_user_interaction(self) -> bool:
         return False
 
-    def _click_editable(self, click_target):
+    def _click_editable(self, click_target: usertypes.ClickTarget) -> None:
         ok = self._elem.evaluateJavaScript('this.focus(); true;')
         if ok:
             self._move_text_cursor()
@@ -328,7 +339,7 @@ class WebKitElement(webelem.AbstractWebElement):
             log.webelem.debug("Failed to focus via JS, falling back to event")
             self._click_fake_event(click_target)
 
-    def _click_js(self, click_target):
+    def _click_js(self, click_target: usertypes.ClickTarget) -> None:
         settings = QWebSettings.globalSettings()
         attribute = QWebSettings.JavascriptCanOpenWindows
         could_open_windows = settings.testAttribute(attribute)
@@ -339,12 +350,12 @@ class WebKitElement(webelem.AbstractWebElement):
             log.webelem.debug("Failed to click via JS, falling back to event")
             self._click_fake_event(click_target)
 
-    def _click_fake_event(self, click_target):
+    def _click_fake_event(self, click_target: usertypes.ClickTarget) -> None:
         self._tab.data.override_target = click_target
         super()._click_fake_event(click_target)
 
 
-def get_child_frames(startframe):
+def get_child_frames(startframe: QWebFrame) -> typing.List[QWebFrame]:
     """Get all children recursively of a given QWebFrame.
 
     Loosely based on http://blog.nextgenetics.net/?e=64
@@ -358,7 +369,7 @@ def get_child_frames(startframe):
     results = []
     frames = [startframe]
     while frames:
-        new_frames = []
+        new_frames = []  # type: typing.List[QWebFrame]
         for frame in frames:
             results.append(frame)
             new_frames += frame.childFrames()
