@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -22,6 +22,7 @@
 import os.path
 import html
 import collections
+import functools
 
 import attr
 from PyQt5.QtCore import (pyqtSlot, pyqtSignal, Qt, QTimer, QDir, QModelIndex,
@@ -34,8 +35,7 @@ from qutebrowser.browser import downloads
 from qutebrowser.config import config
 from qutebrowser.utils import usertypes, log, utils, qtutils, objreg, message
 from qutebrowser.keyinput import modeman
-from qutebrowser.commands import cmdutils, cmdexc
-from qutebrowser.qt import sip
+from qutebrowser.api import cmdutils
 
 
 prompt_queue = None
@@ -117,10 +117,11 @@ class PromptQueue(QObject):
         log.prompt.debug("Popping from queue {}".format(self._queue))
         if self._queue:
             question = self._queue.popleft()
-            if not sip.isdeleted(question):
-                # the question could already be deleted, e.g. by a cancelled
+            if not question.is_aborted:
+                # the question could already be aborted, e.g. by a cancelled
                 # download. See
-                # https://github.com/qutebrowser/qutebrowser/issues/415
+                # https://github.com/qutebrowser/qutebrowser/issues/415 and
+                # https://github.com/qutebrowser/qutebrowser/issues/1249
                 self.ask_question(question, blocking=False)
 
     def shutdown(self):
@@ -329,11 +330,10 @@ class PromptContainer(QWidget):
         log.prompt.debug("Displaying prompt {}".format(prompt))
         self._prompt = prompt
 
+        # If this question was interrupted, we already connected the signal
         if not question.interrupted:
-            # If this question was interrupted, we already connected the signal
             question.aborted.connect(
-                lambda: modeman.leave(self._win_id, prompt.KEY_MODE, 'aborted',
-                                      maybe=True))
+                functools.partial(self._on_aborted, prompt.KEY_MODE))
         modeman.enter(self._win_id, prompt.KEY_MODE, 'question asked')
 
         self.setSizePolicy(prompt.sizePolicy())
@@ -343,6 +343,15 @@ class PromptContainer(QWidget):
         prompt.setFocus()
         self.update_geometry.emit()
 
+    @pyqtSlot()
+    def _on_aborted(self, key_mode):
+        """Leave KEY_MODE whenever a prompt is aborted."""
+        try:
+            modeman.leave(self._win_id, key_mode, 'aborted', maybe=True)
+        except objreg.RegistryUnavailableError:
+            # window was deleted: ignore
+            pass
+
     @pyqtSlot(usertypes.KeyMode)
     def _on_prompt_done(self, key_mode):
         """Leave the prompt mode in this window if a question was answered."""
@@ -384,14 +393,15 @@ class PromptContainer(QWidget):
         try:
             done = self._prompt.accept(value)
         except Error as e:
-            raise cmdexc.CommandError(str(e))
+            raise cmdutils.CommandError(str(e))
         if done:
             message.global_bridge.prompt_done.emit(self._prompt.KEY_MODE)
             question.done()
 
     @cmdutils.register(instance='prompt-container', scope='window',
                        modes=[usertypes.KeyMode.prompt], maxsplit=0)
-    def prompt_open_download(self, cmdline: str = None, pdfjs=False):
+    def prompt_open_download(self, cmdline: str = None,
+                             pdfjs: bool = False) -> None:
         """Immediately open a download.
 
         If no specific command is given, this will use the system's default
