@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -21,21 +21,16 @@
 
 import functools
 import os
-import signal
 import traceback
 
-try:
-    import hunter
-except ImportError:
-    hunter = None
-
 from PyQt5.QtCore import QUrl
 # so it's available for :debug-pyeval
 from PyQt5.QtWidgets import QApplication  # pylint: disable=unused-import
 
 from qutebrowser.browser import qutescheme
 from qutebrowser.utils import log, objreg, usertypes, message, debug, utils
-from qutebrowser.commands import cmdutils, runners, cmdexc
+from qutebrowser.commands import runners
+from qutebrowser.api import cmdutils
 from qutebrowser.config import config, configdata
 from qutebrowser.misc import consolewidget
 from qutebrowser.utils.version import pastebin_version
@@ -43,8 +38,8 @@ from qutebrowser.qt import sip
 
 
 @cmdutils.register(maxsplit=1, no_cmd_split=True, no_replace_variables=True)
-@cmdutils.argument('win_id', win_id=True)
-def later(ms: int, command, win_id):
+@cmdutils.argument('win_id', value=cmdutils.Value.win_id)
+def later(ms: int, command: str, win_id: int) -> None:
     """Execute a command after some time.
 
     Args:
@@ -52,7 +47,7 @@ def later(ms: int, command, win_id):
         command: The command to run, with optional args.
     """
     if ms < 0:
-        raise cmdexc.CommandError("I can't run something in the past!")
+        raise cmdutils.CommandError("I can't run something in the past!")
     commandrunner = runners.CommandRunner(win_id)
     app = objreg.get('app')
     timer = usertypes.Timer(name='later', parent=app)
@@ -61,8 +56,8 @@ def later(ms: int, command, win_id):
         try:
             timer.setInterval(ms)
         except OverflowError:
-            raise cmdexc.CommandError("Numeric argument is too large for "
-                                      "internal int representation.")
+            raise cmdutils.CommandError("Numeric argument is too large for "
+                                        "internal int representation.")
         timer.timeout.connect(
             functools.partial(commandrunner.run_safely, command))
         timer.timeout.connect(timer.deleteLater)
@@ -73,9 +68,9 @@ def later(ms: int, command, win_id):
 
 
 @cmdutils.register(maxsplit=1, no_cmd_split=True, no_replace_variables=True)
-@cmdutils.argument('win_id', win_id=True)
-@cmdutils.argument('count', count=True)
-def repeat(times: int, command, win_id, count=None):
+@cmdutils.argument('win_id', value=cmdutils.Value.win_id)
+@cmdutils.argument('count', value=cmdutils.Value.count)
+def repeat(times: int, command: str, win_id: int, count: int = None) -> None:
     """Repeat a given command.
 
     Args:
@@ -87,16 +82,17 @@ def repeat(times: int, command, win_id, count=None):
         times *= count
 
     if times < 0:
-        raise cmdexc.CommandError("A negative count doesn't make sense.")
+        raise cmdutils.CommandError("A negative count doesn't make sense.")
     commandrunner = runners.CommandRunner(win_id)
     for _ in range(times):
         commandrunner.run_safely(command)
 
 
 @cmdutils.register(maxsplit=1, no_cmd_split=True, no_replace_variables=True)
-@cmdutils.argument('win_id', win_id=True)
-@cmdutils.argument('count', count=True)
-def run_with_count(count_arg: int, command, win_id, count=1):
+@cmdutils.argument('win_id', value=cmdutils.Value.win_id)
+@cmdutils.argument('count', value=cmdutils.Value.count)
+def run_with_count(count_arg: int, command: str, win_id: int,
+                   count: int = 1) -> None:
     """Run a command with the given count.
 
     If run_with_count itself is run with a count, it multiplies count_arg.
@@ -109,60 +105,12 @@ def run_with_count(count_arg: int, command, win_id, count=1):
     runners.CommandRunner(win_id).run(command, count_arg * count)
 
 
-@cmdutils.register()
-def message_error(text):
-    """Show an error message in the statusbar.
-
-    Args:
-        text: The text to show.
-    """
-    message.error(text)
-
-
-@cmdutils.register()
-@cmdutils.argument('count', count=True)
-def message_info(text, count=1):
-    """Show an info message in the statusbar.
-
-    Args:
-        text: The text to show.
-        count: How many times to show the message
-    """
-    for _ in range(count):
-        message.info(text)
-
-
-@cmdutils.register()
-def message_warning(text):
-    """Show a warning message in the statusbar.
-
-    Args:
-        text: The text to show.
-    """
-    message.warning(text)
-
-
 @cmdutils.register()
 def clear_messages():
     """Clear all message notifications."""
     message.global_bridge.clear_messages.emit()
 
 
-@cmdutils.register(debug=True)
-@cmdutils.argument('typ', choices=['exception', 'segfault'])
-def debug_crash(typ='exception'):
-    """Crash for debugging purposes.
-
-    Args:
-        typ: either 'exception' or 'segfault'.
-    """
-    if typ == 'segfault':
-        os.kill(os.getpid(), signal.SIGSEGV)
-        raise Exception("Segfault failed (wat.)")
-    else:
-        raise Exception("Forced crash")
-
-
 @cmdutils.register(debug=True)
 def debug_all_objects():
     """Print a list of  all objects to the debug log."""
@@ -218,22 +166,6 @@ def debug_console():
         con_widget.show()
 
 
-@cmdutils.register(debug=True, maxsplit=0, no_cmd_split=True)
-def debug_trace(expr=""):
-    """Trace executed code via hunter.
-
-    Args:
-        expr: What to trace, passed to hunter.
-    """
-    if hunter is None:
-        raise cmdexc.CommandError("You need to install 'hunter' to use this "
-                                  "command!")
-    try:
-        eval('hunter.trace({})'.format(expr))
-    except Exception as e:
-        raise cmdexc.CommandError("{}: {}".format(e.__class__.__name__, e))
-
-
 @cmdutils.register(maxsplit=0, debug=True, no_cmd_split=True)
 def debug_pyeval(s, file=False, quiet=False):
     """Evaluate a python string and display the results as a web page.
@@ -250,7 +182,7 @@ def debug_pyeval(s, file=False, quiet=False):
             with open(path, 'r', encoding='utf-8') as f:
                 s = f.read()
         except OSError as e:
-            raise cmdexc.CommandError(str(e))
+            raise cmdutils.CommandError(str(e))
         try:
             exec(s)
             out = "No error"
@@ -269,7 +201,7 @@ def debug_pyeval(s, file=False, quiet=False):
     else:
         tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                     window='last-focused')
-        tabbed_browser.openurl(QUrl('qute://pyeval'), newtab=True)
+        tabbed_browser.load_url(QUrl('qute://pyeval'), newtab=True)
 
 
 @cmdutils.register(debug=True)
@@ -286,8 +218,8 @@ def debug_set_fake_clipboard(s=None):
 
 
 @cmdutils.register()
-@cmdutils.argument('win_id', win_id=True)
-@cmdutils.argument('count', count=True)
+@cmdutils.argument('win_id', value=cmdutils.Value.win_id)
+@cmdutils.argument('count', value=cmdutils.Value.count)
 def repeat_command(win_id, count=None):
     """Repeat the last executed command.
 
@@ -296,22 +228,23 @@ def repeat_command(win_id, count=None):
     """
     mode_manager = objreg.get('mode-manager', scope='window', window=win_id)
     if mode_manager.mode not in runners.last_command:
-        raise cmdexc.CommandError("You didn't do anything yet.")
+        raise cmdutils.CommandError("You didn't do anything yet.")
     cmd = runners.last_command[mode_manager.mode]
     commandrunner = runners.CommandRunner(win_id)
     commandrunner.run(cmd[0], count if count is not None else cmd[1])
 
 
 @cmdutils.register(debug=True, name='debug-log-capacity')
-def log_capacity(capacity: int):
+def log_capacity(capacity: int) -> None:
     """Change the number of log lines to be stored in RAM.
 
     Args:
        capacity: Number of lines for the log.
     """
     if capacity < 0:
-        raise cmdexc.CommandError("Can't set a negative log capacity!")
+        raise cmdutils.CommandError("Can't set a negative log capacity!")
     else:
+        assert log.ram_handler is not None
         log.ram_handler.change_log_capacity(capacity)
 
 
@@ -319,18 +252,19 @@ def log_capacity(capacity: int):
 @cmdutils.argument('level', choices=sorted(
     (level.lower() for level in log.LOG_LEVELS),
     key=lambda e: log.LOG_LEVELS[e.upper()]))
-def debug_log_level(level: str):
+def debug_log_level(level: str) -> None:
     """Change the log level for console logging.
 
     Args:
         level: The log level to set.
     """
     log.change_console_formatter(log.LOG_LEVELS[level.upper()])
+    assert log.console_handler is not None
     log.console_handler.setLevel(log.LOG_LEVELS[level.upper()])
 
 
 @cmdutils.register(debug=True)
-def debug_log_filter(filters: str):
+def debug_log_filter(filters: str) -> None:
     """Change the log filter for console logging.
 
     Args:
@@ -338,23 +272,23 @@ def debug_log_filter(filters: str):
                  clear any existing filters.
     """
     if log.console_filter is None:
-        raise cmdexc.CommandError("No log.console_filter. Not attached "
-                                  "to a console?")
+        raise cmdutils.CommandError("No log.console_filter. Not attached "
+                                    "to a console?")
 
     if filters.strip().lower() == 'none':
         log.console_filter.names = None
         return
 
     if not set(filters.split(',')).issubset(log.LOGGER_NAMES):
-        raise cmdexc.CommandError("filters: Invalid value {} - expected one "
-                                  "of: {}".format(filters,
-                                                  ', '.join(log.LOGGER_NAMES)))
+        raise cmdutils.CommandError("filters: Invalid value {} - expected one "
+                                    "of: {}".format(
+                                        filters, ', '.join(log.LOGGER_NAMES)))
 
     log.console_filter.names = filters.split(',')
 
 
 @cmdutils.register()
-@cmdutils.argument('current_win_id', win_id=True)
+@cmdutils.argument('current_win_id', value=cmdutils.Value.win_id)
 def window_only(current_win_id):
     """Close all windows except for the current one."""
     for win_id, window in objreg.window_registry.items():
@@ -368,13 +302,7 @@ def window_only(current_win_id):
 
 
 @cmdutils.register()
-def nop():
-    """Do nothing."""
-    pass
-
-
-@cmdutils.register()
-@cmdutils.argument('win_id', win_id=True)
+@cmdutils.argument('win_id', value=cmdutils.Value.win_id)
 def version(win_id, paste=False):
     """Show version information.
 
@@ -383,7 +311,7 @@ def version(win_id, paste=False):
     """
     tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                 window=win_id)
-    tabbed_browser.openurl(QUrl('qute://version'), newtab=True)
+    tabbed_browser.load_url(QUrl('qute://version'), newtab=True)
 
     if paste:
         pastebin_version()
